"""
Create table from a MySQL or Rdb database from a T4 csv file
"""
#
# usage:
#    python t4_tables_v3.py [options] input_file output_file
#    where
#         input_file: file containing T4 data
#         output_file: file receiving SQL statements
#
#
import sys, csv
from optparse import OptionParser
# import pprint

MYSQL_MAX_COLNAME_SIZE = 63
RDB_MAX_COLNAME_SIZE = 31

def normalize_table_name(table_name):
    "Remove ':' and replace '.' by '_' into table_name"
    return table_name.replace('.', '_').replace(':', '')

def extract_names(name):
    """
    return
            - None if Sample Time column
        or
            - table name, column name
    """
    name = name.lstrip()
    fclass, fname = name.split(']')
    if fname == 'Sample Time': # new table
        return None
    else:
        return normalize_table_name(fclass.replace('[','')), name

def gen_security(fres, options, name):
    if options.writer:
        print >> fres, 'grant select, insert, update, delete'
        print >> fres, '    on table "%s" to %s position 2;' % \
              (name, options.writer)
        posread = 3
    else:
        posread = 2
    if options.reader:
        print >> fres, 'grant select'
        print >> fres, '    on table "%s" to %s position %d;' % \
              (name, options.reader, posread)

class Table:
    """
    Table class
    """
    def __init__(self, name, options, columns_name = None, backend = None):
        self.name = name
        self.columns_name = []
        self.repository = []
        self.colseq = 0
        self.long_colname = options.long_name
        if columns_name:
            for colname in columns_name:
                self.add_column_name(colname)
        self.backend = backend
    def add_column_name(self, column_name):
        # rdb maximum column name size is 31 characters
        if self.backend == 'mysql':
            max_colname_size = MYSQL_MAX_COLNAME_SIZE
        else:
            max_colname_size = RDB_MAX_COLNAME_SIZE
        column_name = column_name.rstrip()
        colname = column_name
        if not self.long_colname:
            self.colseq += 1
            column_name = "C%03d" % (self.colseq,)
        else:
            column_name = column_name[:max_colname_size]
        if self.long_colname and colname != column_name:
            print '\ntruncate\n"%s"\n  into\n"%s"' % (colname, column_name)
        self.columns_name.append(column_name)
        self.repository.append([column_name, colname])
    def generate_create(self, fres, options):
        print >> fres
        if options.with_drop_table:
            if options.backend == 'mysql':
                print >> fres, 'drop table if exists %s;' % self.name
            else:
                print >> fres, 'drop table "%s";' % self.name.upper()
        if options.backend == 'rdb':
            print >> fres, 'create table "%s" (' % self.name.upper()
            print >> fres, '    node_id integer NOT NULL not deferrable,'
        else:
            print >> fres, 'create table %s (' % self.name
            print >> fres, '    node_id integer NOT NULL,'
        print >> fres, '    sample_time timestamp(0),'
        for colname in self.columns_name:
            print >> fres, '    "%s" float,' % colname
        print >> fres, '    PRIMARY KEY (node_id, sample_time)',
        if options.backend == 'mysql':
            print >> fres
            print >> fres, ') type=InnoDB CHECKSUM=1 ROW_FORMAT=COMPRESSED;'
        elif options.backend == 'rdb':
            print >> fres, ' not deferrable'
            print >> fres, ');'
            gen_security(fres, options, self.name.upper())
            print >> fres, 'create unique index %s_idx1 on "%s" (' %\
                  (self.name, self.name.upper())
            print >> fres, '    node_id, sample_time);'
    def store_repository(self, fres, options):
        print >> fres
        print >> fres, "delete from repository where table_name = '%s';" \
              % (self.name,)
        for (c, v) in self.repository:
            req = "insert into repository (table_name, column_name, item_name)"
            req += " values ('%s', '%s', '%s');"
            print >> fres, req % (self.name, c, v)

def main():
    "Main routine"
    usage = "usage: python %prog [options] input_file output_file"
    parser = OptionParser(usage = usage)
    parser.add_option("-d", "--drop_table", action="store_true",
                      dest="with_drop_table", default=False,
                      help="Generate drop table before create table")
    parser.add_option("-n", "--with_table_node", action="store_true",
                      dest="with_table_node", default=False,
                      help="Generate table node")
    parser.add_option("-m", "--with_table_modes", action="store_true",
                      dest="with_table_modes", default=False,
                      help="Generate table mon_modes")
    parser.add_option("-r", "--with_table_repository", action="store_true",
                      dest="with_table_repository", default=False,
                      help="Generate table repository")
    parser.add_option("-c", "--cpu_count", action="store",
                      dest="cpu_count", type="int", default=32,
                      help="Nb of CPU needed in table mon_modes, default 32")
    parser.add_option("-b", "--backend", action="store", type="choice",
                      choices=["rdb", "mysql"], default="mysql",
                      help="Choice storage backend")
    parser.add_option("-s", "--schema", action="store", type="string",
                      dest="schema", default="t4",
                      help="name of rdb schema or mysql database to create")
    parser.add_option("-l", "--long_name", action="store_true",
                      dest="long_name", default=False,
                      help="use long column names")
    parser.add_option("-W", "--writer", action="store",type="string", 
                      dest="writer", default=None)
    parser.add_option("-R", "--reader", action="store",type="string", 
                      dest="reader", default=None)
    (options, args) = parser.parse_args()

    input_file = args[0]
    output_file = args[1]

    fres = open(output_file, 'w')
    
    reader = csv.reader(file(input_file))

    row = reader.next()
    node = row[0]

    row = reader.next()
    date = row[0]

    row = reader.next()

    print >> fres, '-- Node: %s Date: %s' % (node, date)
    print >> fres
    if options.backend == 'mysql':
        print >> fres, "SET SESSION sql_mode='ansi';"
        print >> fres, "create database %s;" % (options.schema.lower(),)
        print >> fres, "use %s;" % (options.schema.upper(),)
    elif options.backend == 'rdb':
        print >> fres, "-- create database filename db multischema is on;"
        print >> fres, "create schema %s;" % (options.schema.lower(),)
        print >> fres, "set schema '%s';" % (options.schema.lower(),)
        print >> fres, "set quoting rules 'SQL92';"
    print >> fres

    if options.backend == 'mysql':
        print >> fres, 'drop table if exists t4_version;'
        print >> fres, 'create table t4_version(major integer, minor integer'
        print >> fres, ') type=InnoDB CHECKSUM=1 ROW_FORMAT=COMPRESSED;'
    else:
        print >> fres, 'drop table t4_version;'
        print >> fres, 'create table t4_version(major integer, minor integer'
        print >> fres, ');'
        gen_security(fres, options, 'T4_VERSION')

    print >> fres, 'insert into t4_version values (3, 0);'

    if options.with_table_node:
        if options.with_drop_table:
            if options.backend == 'mysql':
                print >> fres, 'drop table if exists node;'
            elif options.backend == 'rdb':
                print >> fres,  'drop table node;'
        print >> fres, 'create table node ('
        if options.backend == 'mysql':
            print >> fres, '    node_id integer NOT NULL auto_increment,'
        elif options.backend == 'rdb':
            print >> fres, '    node_id integer identity,'
        print >> fres, '    node_name char(32),'
        print >> fres, '    PRIMARY KEY (node_id)',
        if options.backend == 'mysql':
            print >> fres
        elif options.backend == 'rdb':
            print >> fres, ' not deferrable'
        if options.backend == 'mysql':
            print >> fres, ') type=InnoDB CHECKSUM=1 ROW_FORMAT=COMPRESSED;'
        else:
            print >> fres, ');'
            gen_security(fres, options, 'NODE')
            print >> fres, 'create unique index node_idx1 on node ('
            print >> fres, '    node_id);'

    if options.with_table_repository:
        if options.with_drop_table:
            if options.backend == 'mysql':
                print >> fres, 'drop table if exists repository;'
            elif options.backend == 'rdb':
                print >> fres,  'drop table repository;'
        print >> fres, 'create table repository ('
        if options.backend == 'mysql':
            print >> fres, '    item_id integer auto_increment,'
        elif options.backend == 'rdb':
            print >> fres, '    item_id integer identity,'
        print >> fres, '    table_name char(64),'
        print >> fres, '    column_name char(64),'
        print >> fres, '    item_name char(256),'
        print >> fres, '    PRIMARY KEY (item_id)',
        if options.backend == 'mysql':
            print >> fres
        elif options.backend == 'rdb':
            print >> fres, ' not deferrable'
        if options.backend == 'mysql':
            print >> fres, ') type=InnoDB CHECKSUM=1 ROW_FORMAT=COMPRESSED;'
            print >> fres, 'create unique index repository_idx2 on repository ('
            print >> fres, '    table_name, column_name);'
        else:
            print >> fres, ');'
            gen_security(fres, options, 'REPOSITORY')
            print >> fres, 'create unique index repository_idx1 on repository ('
            print >> fres, '    item_id);'
            print >> fres, 'create unique index repository_idx2 on repository ('
            print >> fres, '    table_name, column_name);'


    header = row = reader.next()

    tables = {}
    for i in range(len(header)):
        info = extract_names(header[i])
        if info == None:
            continue
        elif info[1].startswith("[MON.MODES]Cpu "):
            continue
        else:
            key = info[0].lower()
            if not tables.has_key(key):
                tables[key] = Table(key, options, backend = options.backend)
            tables[key].add_column_name(info[1].lstrip())

    tbl = tables['mon_modes'] = Table('mon_modes', options,
                                      backend = options.backend)
    for i in range(options.cpu_count):
        tbl.add_column_name('[MON.MODES]Cpu %02d Inter mode' % i)
        tbl.add_column_name('[MON.MODES]Cpu %02d MpSync mode' % i)
        tbl.add_column_name('[MON.MODES]Cpu %02d Kernel mode' % i)
        tbl.add_column_name('[MON.MODES]Cpu %02d Exec mode' % i)
        tbl.add_column_name('[MON.MODES]Cpu %02d Super mode' % i)
        tbl.add_column_name('[MON.MODES]Cpu %02d User mode' % i)
        tbl.add_column_name('[MON.MODES]Cpu %02d Busy' % i)
        tbl.add_column_name('[MON.MODES]Cpu %02d Idle' % i)

    for table in tables.itervalues():
        if table.name != 'mon_modes' or options.with_table_modes: 
            table.generate_create(fres, options)

    for table in tables.itervalues():
        if table.name != 'mon_modes' or options.with_table_modes: 
            table.store_repository(fres, options)
            
    if options.backend == 'rdb':
        print >> fres, "-- commit work;"
    row = reader.next()
    # pprint.pprint(row)
    for i in range(len(row)):
        print >> fres, '-- %d %s: %s' % (len(header[i]), header[i], row[i])

    #for row in reader:
    #    for i in range(len(row)):
    #        print '%d %s: %s' % (len(header[i]), header[i], row[i])
    #    print
    fres.close()

if __name__ == '__main__':
    main()
